"use strict";

exports.__esModule = true;

var _Formatter = require("../core/Formatter");

var _Formatter2 = _interopRequireDefault(_Formatter);

var _Tokenizer = require("../core/Tokenizer");

var _Tokenizer2 = _interopRequireDefault(_Tokenizer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var reservedWords = ["as", "asc", "auto_increment", "between", "case", "character set", "charset", "comment", "contains", "current_timestamp", "current_date", "count", "coalesce", "database", "databases", "default", "delete", "desc", "describe", "distinct", "else", "end", "engine", "exists", "explain", "fields", "file", "foreign", "full", "function", "global", "grant", "grants", "group_concat", "hour", "identified", "if", "ifnull", "in", "index", "indexes", "infile", "insert", "interval", "into", "invoker", "is", "key", "keys", "kill", "like", "match", "minute", "min", "max", "modify", "month", "names", "now()", "null", "partition", "partitions", "regexp", "rename", "replace", "replication", "reset", "rlike", "row", "rows", "row_format", "second", "storage", "string", "sum", "table", "tables", "temporary", "terminated", "then", "to", "true", "truncate", "type", "types", "uncommitted", "unique", "unsigned", "usage", "use", "using", "variables", "view", "when", "with"];

var reservedToplevelWords = ["delete from", "except", "group by", "order by", "having", "intersect", "modify", "select", "update", "values"];

var unionWords = ["union all", "union"];

var reservedToplevelInLineWords = ["use", "drop table", "create table", "from", "where", "limit", "create", "insert overwrite", "insert into", "inner join", "full join", "full outer join", "join", "left join", "left outer join", "outer join", "right join", "right outer join", "on"];

var reservedNewlineWords = ["and", "or", "partitioned by", "row format", "fields terminated by", "lines terminated by", "stored as", "tblproperties", "alter table", "add jar", "after", "alter column", "cross apply", "cross join", "set", "when", "else"];

var tokenizer = void 0;

var SparkSqlFormatter = function () {
    /**
     * @param {Object} cfg Different set of configurations
     */
    function SparkSqlFormatter(cfg) {
        _classCallCheck(this, SparkSqlFormatter);

        this.cfg = cfg;
    }

    /**
     * Format the whitespace in a SparkSQL string to make it easier to read
     *
     * @param {String} query The SparkSQL string
     * @return {String} formatted string
     */


    SparkSqlFormatter.prototype.format = function format(query) {
        if (!tokenizer) {
            tokenizer = new _Tokenizer2["default"]({
                reservedWords: reservedWords,
                reservedToplevelWords: reservedToplevelWords,
                reservedNewlineWords: reservedNewlineWords,
                reservedToplevelInLineWords: reservedToplevelInLineWords,
                unionWords: unionWords,
                stringTypes: ["\"\"", "N''", "''", "``", "[]"],
                openParens: ["(", "CASE"],
                closeParens: [")", "END"],
                indexedPlaceholderTypes: ["?"],
                lineCommentTypes: ["#", "--"]
            });
        }
        return new _Formatter2["default"](this.cfg, tokenizer).format(query);
    };

    return SparkSqlFormatter;
}();

exports["default"] = SparkSqlFormatter;
module.exports = exports["default"];